Method: Gecode::Mixin#track_variable
- Defined in:
- lib/gecoder/interface/mixin.rb
#track_variable(variable) ⇒ Object
Starts tracking a variable that depends on the space. All variables created should call this method for their respective models.
271 272 273 |
# File 'lib/gecoder/interface/mixin.rb', line 271 def track_variable(variable) #:nodoc: (@gecoder_mixin_variables ||= []) << variable end |